@@ -10,6 +10,7 @@
 """
 import datetime
 import inspect
+import uuid
 
 from dateutil.parser import parse as parse_datetime
 from sqlalchemy import Date
@@ -24,15 +25,21 @@ from sqlalchemy.orm.attributes import InstrumentedAttribute
 from sqlalchemy.orm.attributes import QueryableAttribute
 from sqlalchemy.orm.exc import UnmappedInstanceError
 from sqlalchemy.orm.query import Query
+from sqlalchemy.orm.util import class_mapper
 from sqlalchemy.sql import func
 from sqlalchemy.sql.expression import _BinaryExpression
+from sqlalchemy.sql.expression import ColumnElement
 
 #: Names of attributes which should definitely not be considered relations when
 #: dynamically computing a list of relations of a SQLAlchemy model.
-BLACKLIST = ('query', 'query_class', '_sa_class_manager',
-             '_decl_class_registry')
+RELATION_BLACKLIST = ('query', 'query_class', '_sa_class_manager',
+                      '_decl_class_registry')
 
 
+#: Names of columns which should definitely not be considered user columns to
+#: be included in a dictionary representation of a model.
+COLUMN_BLACKLIST = ('_sa_polymorphic_on', )
+
 #: Types which should be considered columns of a model when iterating over all
 #: attributes of a model class.
 COLUMN_TYPES = (InstrumentedAttribute, hybrid_property)
@@ -110,7 +117,8 @@ def get_columns(model):
 
 def get_relations(model):
     """Returns a list of relation names of `model` (as a list of strings)."""
-    return [k for k in dir(model) if not (k.startswith('__') or k in BLACKLIST)
+    return [k for k in dir(model)
+            if not (k.startswith('__') or k in RELATION_BLACKLIST)
             and get_related_model(model, k)]
 
 
@@ -123,11 +131,22 @@ def get_related_model(model, relationname):
     attr = getattr(model, relationname)
     if relationname in cols and isinstance(attr.property, RelProperty):
         return cols[relationname].property.mapper.class_
-    elif isinstance(attr, AssociationProxy):
-        if hasattr(attr.remote_attr.property, 'mapper'):
-            return attr.remote_attr.property.mapper.class_
-        elif hasattr(attr.remote_attr.property, 'parent'):
-            return attr.remote_attr.property.parent.class_
+    if isinstance(attr, AssociationProxy):
+        return get_related_association_proxy_model(attr)
+    return None
+
+
+def get_related_association_proxy_model(attr):
+    """Returns the model class specified by the given SQLAlchemy relation
+    attribute, or ``None`` if no such class can be inferred.
+
+    `attr` must be a relation attribute corresponding to an association proxy.
+
+    """
+    prop = attr.remote_attr.property
+    for attribute in ('mapper', 'parent'):
+        if hasattr(prop, attribute):
+            return getattr(prop, attribute).class_
     return None
 
 
@@ -147,12 +166,15 @@ def is_date_field(model, fieldname):
 
     """
     field = getattr(model, fieldname)
-    if isinstance(field, AssociationProxy):
-        field = field.remote_attr
-    prop = field.property
-    if isinstance(prop, RelProperty):
-        return False
-    fieldtype = prop.columns[0].type
+    if isinstance(field, ColumnElement):
+        fieldtype = field.type
+    else:
+        if isinstance(field, AssociationProxy):
+            field = field.remote_attr
+        prop = field.property
+        if isinstance(prop, RelProperty):
+            return False
+        fieldtype = prop.columns[0].type
     return isinstance(fieldtype, Date) or isinstance(fieldtype, DateTime)
 
 
@@ -208,13 +230,26 @@ def is_like_list(instance, relation):
     return isinstance(related_value, AssociationProxy)
 
 
+def is_mapped_class(cls):
+    try:
+        class_mapper(cls)
+        return True
+    except:
+        return False
+
 # This code was adapted from :meth:`elixir.entity.Entity.to_dict` and
 # http://stackoverflow.com/q/1958219/108197.
 def to_dict(instance, deep=None, exclude=None, include=None,
-            exclude_relations=None, include_relations=None):
+            exclude_relations=None, include_relations=None,
+            include_methods=None):
     """Returns a dictionary representing the fields of the specified `instance`
     of a SQLAlchemy model.
 
+    The returned dictionary is suitable as an argument to
+    :func:`flask.jsonify`; :class:`datetime.date` and :class:`uuid.UUID`
+    objects are converted to string representations, so no special JSON encoder
+    behavior is required.
+
     `deep` is a dictionary containing a mapping from a relation name (for a
     relation of `instance`) to either a list or a dictionary. This is a
     recursive structure which represents the `deep` argument when calling
@@ -242,6 +277,9 @@ def to_dict(instance, deep=None, exclude=None, include=None,
     names of fields on the related model which should be included in the
     returned dictionary; `exclude_relations` is similar.
 
+    `include_methods` is a list mapping strings to method names which will
+    be called and their return values added to the returned dictionary.
+
     """
     if (exclude is not None or exclude_relations is not None) and \
             (include is not None or include_relations is not None):
@@ -261,13 +299,24 @@ def to_dict(instance, deep=None, exclude=None, include=None,
     elif include is not None:
         columns = (c for c in columns if c in include)
     # create a dictionary mapping column name to value
-    result = dict((col, getattr(instance, col)) for col in columns)
-    # Convert datetime and date objects to ISO 8601 format.
-    #
-    # TODO We can get rid of this when issue #33 is resolved.
+    result = dict((col, getattr(instance, col)) for col in columns
+                  if not (col.startswith('__') or col in COLUMN_BLACKLIST))
+    # add any included methods
+    if include_methods is not None:
+        result.update(dict((method, getattr(instance, method)())
+                           for method in include_methods
+                           if not '.' in method))
+    # Check for objects in the dictionary that may not be serializable by
+    # default. Specifically, convert datetime and date objects to ISO 8601
+    # format, and convert UUID objects to hexadecimal strings.
     for key, value in result.items():
+        # TODO We can get rid of this when issue #33 is resolved.
         if isinstance(value, datetime.date):
             result[key] = value.isoformat()
+        elif isinstance(value, uuid.UUID):
+            result[key] = str(value)
+        elif is_mapped_class(type(value)):
+            result[key] = to_dict(value)
     # recursively call _to_dict on each of the `deep` relations
     deep = deep or {}
     for relation, rdeep in deep.iteritems():
@@ -286,9 +335,15 @@ def to_dict(instance, deep=None, exclude=None, include=None,
         elif (include_relations is not None and
               relation in include_relations):
             newinclude = include_relations[relation]
+        # Determine the included methods for the related model.
+        newmethods = None
+        if include_methods is not None:
+            newmethods = [method.split('.', 1)[1] for method in include_methods
+                        if method.split('.', 1)[0] == relation]
         if is_like_list(instance, relation):
             result[relation] = [to_dict(inst, rdeep, exclude=newexclude,
-                                        include=newinclude)
+                                        include=newinclude,
+                                        include_methods=newmethods)
                                 for inst in relatedvalue]
             continue
         # If the related value is dynamically loaded, resolve the query to get
@@ -296,7 +351,8 @@ def to_dict(instance, deep=None, exclude=None, include=None,
         if isinstance(relatedvalue, Query):
             relatedvalue = relatedvalue.one()
         result[relation] = to_dict(relatedvalue, rdeep, exclude=newexclude,
-                                   include=newinclude)
+                                   include=newinclude,
+                                   include_methods=newmethods)
     return result
 
 
